@@ -81,12 +81,34 @@ public:
     AVFrame *m_avFrame;
 };
 
+static AVPixelFormat convertQImageFormatToAVPixelFormat(QImage::Format format)
+{
+    // Listing those handed by SpaToQImageFormat
+    switch (format) {
+    case QImage::Format_BGR888:
+        return AV_PIX_FMT_BGR24;
+    case QImage::Format_RGBX8888:
+    case QImage::Format_RGBA8888_Premultiplied:
+        return AV_PIX_FMT_RGBA;
+    case QImage::Format_RGB32:
+        return AV_PIX_FMT_RGB32;
+    default:
+        qDebug() << "Unexpected pixel format" << format;
+        return AV_PIX_FMT_RGB32;
+    }
+}
+
+Q_DECLARE_METATYPE(std::optional<int>);
+Q_DECLARE_METATYPE(std::optional<std::chrono::nanoseconds>);
+
 PipeWireRecord::PipeWireRecord(QObject *parent)
     : QObject(parent)
     , d(new PipeWireRecordPrivate)
 {
     d->m_encoder = "libvpx";
     av_log_set_level(AV_LOG_DEBUG);
+    qRegisterMetaType<std::optional<int>>();
+    qRegisterMetaType<std::optional<std::chrono::nanoseconds>>();
 }
 
 PipeWireRecord::~PipeWireRecord()
@@ -208,8 +230,8 @@ void PipeWireRecordProduce::stateChanged(pw_stream_state state)
 
     disconnect(m_stream.data(), &PipeWireSourceStream::frameReceived, this, &PipeWireRecordProduce::processFrame);
     if (m_writeThread) {
-        m_writeThread->drain();
-        bool done = QThreadPool::globalInstance()->waitForDone(-1);
+        m_writeThread->quit();
+        bool done = m_writeThread->wait();
         Q_ASSERT(done);
     }
 
@@ -228,6 +250,15 @@ QString PipeWireRecord::extension()
     return QStringLiteral("webm");
 }
 
+QString PipeWireRecord::currentExtension() const
+{
+    static QHash<QByteArray, QString> s_extensions = {
+        {"libx264", QStringLiteral("mp4")},
+        {"libvpx", QStringLiteral("webm")},
+    };
+    return s_extensions.value(d->m_encoder, QStringLiteral("mkv"));
+}
+
 void PipeWireRecordProduce::setupStream()
 {
     qCDebug(PIPEWIRERECORD_LOGGING) << "Setting up stream";
@@ -265,7 +296,7 @@ void PipeWireRecordProduce::setupStream()
     m_avCodecContext->width = size.width();
     m_avCodecContext->height = size.height();
     m_avCodecContext->max_b_frames = 1;
-    m_avCodecContext->gop_size = 1;
+    m_avCodecContext->gop_size = 100;
     if (m_codec->pix_fmts && m_codec->pix_fmts[0] > 0) {
         m_avCodecContext->pix_fmt = m_codec->pix_fmts[0];
     } else {
@@ -274,10 +305,17 @@ void PipeWireRecordProduce::setupStream()
     m_avCodecContext->time_base = AVRational{1, 1000};
 
     AVDictionary *options = nullptr;
-    av_dict_set_int(&options, "threads", 4, 0);
-    av_dict_set(&options, "preset", "ultrafast", 0);
+    av_dict_set_int(&options, "threads", qMin(16, QThread::idealThreadCount()), 0);
+    av_dict_set(&options, "preset", "veryfast", 0);
     av_dict_set(&options, "tune-content", "screen", 0);
-    av_dict_set(&options, "deadline", "good", 0);
+    av_dict_set(&options, "deadline", "realtime", 0);
+    // In theory a lower number should be faster, but the opposite seems to be true
+    av_dict_set(&options, "quality", "40", 0);
+    av_dict_set(&options, "cpu-used", "6", 0);
+    // Disable motion estimation, not great while dragging windows but speeds up encoding by an order of magnitude
+    av_dict_set(&options, "flags", "+mv4", 0);
+    // Disable in-loop filtering
+    av_dict_set(&options, "-flags", "+loop", 0);
 
     int ret = avcodec_open2(m_avCodecContext, m_codec, &options);
     if (ret < 0) {
@@ -285,13 +323,6 @@ void PipeWireRecordProduce::setupStream()
         return;
     }
 
-    m_frame.reset(new CustomAVFrame);
-    ret = m_frame->alloc(m_avCodecContext->width, m_avCodecContext->height, m_avCodecContext->pix_fmt);
-    if (ret < 0) {
-        qCWarning(PIPEWIRERECORD_LOGGING) << "Could not allocate raw picture buffer" << av_err2str(ret);
-        return;
-    }
-
     ret = avio_open(&m_avFormatContext->pb, QFile::encodeName(m_output).constData(), AVIO_FLAG_WRITE);
     if (ret < 0) {
         qCWarning(PIPEWIRERECORD_LOGGING) << "Could not open" << m_output << av_err2str(ret);
@@ -318,8 +349,8 @@ void PipeWireRecordProduce::setupStream()
     }
 
     connect(m_stream.data(), &PipeWireSourceStream::frameReceived, this, &PipeWireRecordProduce::processFrame);
-    m_writeThread = new PipeWireRecordWriteThread(&m_bufferNotEmpty, m_avFormatContext, m_avCodecContext);
-    QThreadPool::globalInstance()->start(m_writeThread);
+    m_writeThread = new PipeWireRecordWriteThread(this, m_avFormatContext, m_avCodecContext);
+    m_writeThread->start();
 }
 
 void PipeWireRecordProduce::processFrame(const PipeWireFrame &frame)
@@ -381,23 +412,6 @@ void PipeWireRecordProduce::updateTextureImage(const QImage &image, const PipeWi
     render(frame);
 }
 
-static AVPixelFormat convertQImageFormatToAVPixelFormat(QImage::Format format)
-{
-    // Listing those handed by SpaToQImageFormat
-    switch (format) {
-    case QImage::Format_BGR888:
-        return AV_PIX_FMT_BGR24;
-    case QImage::Format_RGBX8888:
-    case QImage::Format_RGBA8888_Premultiplied:
-        return AV_PIX_FMT_RGBA;
-    case QImage::Format_RGB32:
-        return AV_PIX_FMT_RGB32;
-    default:
-        qDebug() << "Unexpected pixel format" << format;
-        return AV_PIX_FMT_RGB32;
-    }
-}
-
 void PipeWireRecordProduce::render(const PipeWireFrame &frame)
 {
     Q_ASSERT(!m_frameWithoutMetadataCursor.isNull());
@@ -409,53 +423,7 @@ void PipeWireRecordProduce::render(const PipeWireFrame &frame)
         p.drawImage(*m_cursor.position, m_cursor.texture);
     }
 
-    const std::uint8_t *buffers[] = {image.constBits(), nullptr};
-    const int strides[] = {static_cast<int>(image.bytesPerLine()), 0, 0, 0};
-    struct SwsContext *sws_context = nullptr;
-    sws_context = sws_getCachedContext(sws_context,
-                                       image.width(),
-                                       image.height(),
-                                       convertQImageFormatToAVPixelFormat(image.format()),
-                                       m_avCodecContext->width,
-                                       m_avCodecContext->height,
-                                       m_avCodecContext->pix_fmt,
-                                       0,
-                                       nullptr,
-                                       nullptr,
-                                       nullptr);
-    sws_scale(sws_context, buffers, strides, 0, m_avCodecContext->height, m_frame->m_avFrame->data, m_frame->m_avFrame->linesize);
-
-    if (frame.presentationTimestamp.has_value()) {
-        const auto current = std::chrono::duration_cast<std::chrono::milliseconds>(*frame.presentationTimestamp).count();
-        if ((*m_avFormatContext->streams)->start_time == 0) {
-            (*m_avFormatContext->streams)->start_time = current;
-        }
-
-        Q_ASSERT((*m_avFormatContext->streams)->start_time <= current);
-        m_frame->m_avFrame->pts = current - (*m_avFormatContext->streams)->start_time;
-    } else {
-        m_frame->m_avFrame->pts = AV_NOPTS_VALUE;
-    }
-
-    // Let's add a key frame every 100 frames and also the first frame
-    if (frame.sequential && (*frame.sequential == 0 || (*frame.sequential - m_lastKeyFrame) > 100)) {
-        m_frame->m_avFrame->key_frame = 1;
-        m_lastKeyFrame = *frame.sequential;
-    }
-
-    if (m_lastPts > 0 && m_frame->m_avFrame->pts <= m_lastPts) {
-        // Make sure we don't have two frames at the same presentation time
-        m_frame->m_avFrame->pts = m_lastPts + 1;
-    }
-    m_lastPts = m_frame->m_avFrame->pts;
-
-    const int ret = avcodec_send_frame(m_avCodecContext, m_frame->m_avFrame);
-    // qDebug() << "issued" << m_frame->m_avFrame->pts;
-    if (ret < 0) {
-        qCWarning(PIPEWIRERECORD_LOGGING) << "Error sending a frame for encoding:" << av_err2str(ret);
-        return;
-    }
-    m_bufferNotEmpty.wakeAll();
+    Q_EMIT producedFrame(image, frame.sequential, frame.presentationTimestamp);
 }
 
 static void log_packet(const AVFormatContext *fmt_ctx, const AVPacket *pkt)
@@ -476,7 +444,25 @@ static void log_packet(const AVFormatContext *fmt_ctx, const AVPacket *pkt)
 
 void PipeWireRecord::setEncoder(const QByteArray &encoder)
 {
+    if (d->m_encoder == encoder) {
+        return;
+    }
     d->m_encoder = encoder;
+    Q_EMIT encoderChanged();
+}
+
+QByteArray PipeWireRecord::encoder() const
+{
+    return d->m_encoder;
+}
+
+QList<QByteArray> PipeWireRecord::suggestedEncoders() const
+{
+    QList<QByteArray> ret = {"libvpx", "libx264"};
+    std::remove_if(ret.begin(), ret.end(), [](const QByteArray &encoder) {
+        return !avcodec_find_encoder_by_name(encoder.constData());
+    });
+    return ret;
 }
 
 QString PipeWireRecord::output() const
@@ -499,39 +485,94 @@ uint PipeWireRecord::fd() const
     return d->m_fd.value_or(0);
 }
 
-PipeWireRecordWriteThread::PipeWireRecordWriteThread(QWaitCondition *notEmpty, AVFormatContext *avFormatContext, AVCodecContext *avCodecContext)
-    : QRunnable()
+PipeWireRecordWrite::PipeWireRecordWrite(PipeWireRecordProduce *produce, AVFormatContext *avFormatContext, AVCodecContext *avCodecContext)
+    : QObject()
     , m_packet(av_packet_alloc())
     , m_avFormatContext(avFormatContext)
     , m_avCodecContext(avCodecContext)
-    , m_bufferNotEmpty(notEmpty)
 {
+    connect(produce, &PipeWireRecordProduce::producedFrame, this, &PipeWireRecordWrite::addFrame);
 }
 
-PipeWireRecordWriteThread::~PipeWireRecordWriteThread()
+PipeWireRecordWrite::~PipeWireRecordWrite()
 {
+    int ret = av_write_trailer(m_avFormatContext);
+    if (ret < 0) {
+        qCWarning(PIPEWIRERECORD_LOGGING) << "failed to write trailer" << av_err2str(ret);
+    }
     av_packet_free(&m_packet);
 }
 
-void PipeWireRecordWriteThread::run()
+PipeWireRecordWriteThread::PipeWireRecordWriteThread(PipeWireRecordProduce *produce, AVFormatContext *avFormatContext, AVCodecContext *avCodecContext)
+    : QThread(produce)
+    , m_produce(produce)
+    , m_avFormatContext(avFormatContext)
+    , m_avCodecContext(avCodecContext)
 {
-    QMutex mutex;
-    int ret = 0;
-    while (true) {
+}
+
+void PipeWireRecordWrite::addFrame(const QImage &image, std::optional<int> sequential, std::optional<std::chrono::nanoseconds> presentationTimestamp)
+{
+    if (!sws_context || m_lastReceivedSize != image.size()) {
+        sws_context = sws_getCachedContext(sws_context,
+                                           image.width(),
+                                           image.height(),
+                                           convertQImageFormatToAVPixelFormat(image.format()),
+                                           m_avCodecContext->width,
+                                           m_avCodecContext->height,
+                                           m_avCodecContext->pix_fmt,
+                                           0,
+                                           nullptr,
+                                           nullptr,
+                                           nullptr);
+    }
+
+    CustomAVFrame avFrame;
+    int ret = avFrame.alloc(m_avCodecContext->width, m_avCodecContext->height, m_avCodecContext->pix_fmt);
+    if (ret < 0) {
+        qCWarning(PIPEWIRERECORD_LOGGING) << "Could not allocate raw picture buffer" << av_err2str(ret);
+        return;
+    }
+    const std::uint8_t *buffers[] = {image.constBits(), nullptr};
+    const int strides[] = {static_cast<int>(image.bytesPerLine()), 0, 0, 0};
+    sws_scale(sws_context, buffers, strides, 0, m_avCodecContext->height, avFrame.m_avFrame->data, avFrame.m_avFrame->linesize);
+
+    if (presentationTimestamp.has_value()) {
+        const auto current = std::chrono::duration_cast<std::chrono::milliseconds>(*presentationTimestamp).count();
+        if ((*m_avFormatContext->streams)->start_time == 0) {
+            (*m_avFormatContext->streams)->start_time = current;
+        }
+
+        Q_ASSERT((*m_avFormatContext->streams)->start_time <= current);
+        avFrame.m_avFrame->pts = current - (*m_avFormatContext->streams)->start_time;
+    } else {
+        avFrame.m_avFrame->pts = AV_NOPTS_VALUE;
+    }
+
+    // Let's add a key frame every 100 frames and also the first frame
+    if (sequential && (*sequential == 0 || (*sequential - m_lastKeyFrame) > 100)) {
+        avFrame.m_avFrame->key_frame = 1;
+        m_lastKeyFrame = *sequential;
+    }
+
+    if (m_lastPts > 0 && avFrame.m_avFrame->pts <= m_lastPts) {
+        // Make sure we don't have two frames at the same presentation time
+        avFrame.m_avFrame->pts = m_lastPts + 1;
+    }
+    m_lastPts = avFrame.m_avFrame->pts;
+
+    ret = avcodec_send_frame(m_avCodecContext, avFrame.m_avFrame);
+    if (ret < 0) {
+        qCWarning(PIPEWIRERECORD_LOGGING) << "Error sending a frame for encoding:" << av_err2str(ret);
+        return;
+    }
+    for (;;) {
         ret = avcodec_receive_packet(m_avCodecContext, m_packet);
-        if (ret == AVERROR_EOF) {
-            break;
-        } else if (ret == AVERROR(EAGAIN)) {
-            if (m_active) {
-                m_bufferNotEmpty->wait(&mutex);
-            } else {
-                int sent = avcodec_send_frame(m_avCodecContext, nullptr);
-                qCDebug(PIPEWIRERECORD_LOGGING) << "draining" << sent;
+        if (ret < 0) {
+            if (ret != AVERROR_EOF && ret != AVERROR(EAGAIN)) {
+                qCWarning(PIPEWIRERECORD_LOGGING) << "Error encoding a frame: " << av_err2str(ret) << ret;
             }
-            continue;
-        } else if (ret < 0) {
-            qCWarning(PIPEWIRERECORD_LOGGING) << "Error encoding a frame: " << av_err2str(ret);
-            continue;
+            break;
         }
 
         m_packet->stream_index = (*m_avFormatContext->streams)->index;
@@ -540,17 +581,30 @@ void PipeWireRecordWriteThread::run()
         ret = av_interleaved_write_frame(m_avFormatContext, m_packet);
         if (ret < 0) {
             qCWarning(PIPEWIRERECORD_LOGGING) << "Error while writing output packet:" << av_err2str(ret);
-            continue;
         }
-    }
-    ret = av_write_trailer(m_avFormatContext);
-    if (ret < 0) {
-        qCWarning(PIPEWIRERECORD_LOGGING) << "failed to write trailer" << av_err2str(ret);
+        av_packet_unref(m_packet);
     }
 }
 
-void PipeWireRecordWriteThread::drain()
+void PipeWireRecordWriteThread::run()
 {
-    m_active = false;
-    m_bufferNotEmpty->wakeAll();
+    PipeWireRecordWrite writer(m_produce, m_avFormatContext, m_avCodecContext);
+    QThread::exec();
+    AVPacket *pkt = av_packet_alloc();
+    avcodec_send_frame(m_avCodecContext, nullptr);
+
+    for (;;) {
+        if (avcodec_receive_packet(m_avCodecContext, pkt) < 0)
+            break;
+
+        pkt->stream_index = (*m_avFormatContext->streams)->index;
+        av_packet_rescale_ts(pkt, m_avCodecContext->time_base, (*m_avFormatContext->streams)->time_base);
+        log_packet(m_avFormatContext, pkt);
+        int ret = av_interleaved_write_frame(m_avFormatContext, pkt);
+        if (ret < 0) {
+            qCWarning(PIPEWIRERECORD_LOGGING) << "Error while writing output packet:" << av_err2str(ret);
+        }
+        av_packet_unref(pkt);
+    }
+    av_packet_free(&pkt);
 }
